%     Copyright (C) 2023 Jordi Cano. Contact: jordi.cg89@gmail.com
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <https://www.gnu.org/licenses/>

function Analize_simulations()
%Analize_simulations Analizes the simulations corresponding to a population

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
population_name     = 'Population_50000_each.mat';  % initial population name
analize_ToSim_pop   = 0;
simfilesfolder      = 'OUTPUT_1'; % leave empty if the current folder
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

celltypes = {'ENDO','EPI','MID'}; % 0, 1, 2, respectively (according to o'hara code), therefore must add 1 for 1,2,3 in matlab
if analize_ToSim_pop == 0 
    pop_commands_name   = [population_name(1:end-4) '_commands.mat'];
else
    goodfolder          = [population_name(1:end-4) '_plots'];
    pop_commands_name   = [goodfolder '\' population_name(1:end-4) '_partners_toSim_commands.mat'];
end

% prepare things, load pop files
P                   = load(pop_commands_name);
PARS                = P.parameters;
N                   = size(PARS,1);
IDs                 = PARS(:,1);
cells               = PARS(:,2);
genders             = PARS(:,3);
phases              = PARS(:,4);
BCLs                = PARS(:,5);
BEATs               = PARS(:,6);
SVSTs               = PARS(:,7);
SVPs                = PARS(:,8);
scalars             = PARS(:,9:end);
channels            = P.channels;

% load simulations: generate file names
files = cell(N,1);
parfor i = 1:N
    ID          = IDs(i);
    phase       = phases(i);
    celltype    = celltypes{cells(i)+1}; % +1 to correct for matlab indexing of vectors
    
    % convert gender code
    if genders(i) == 1
        gender      = 'm';
    else
        gender      = 'f';
    end
    filename    = ['res_' num2str(ID) '_' celltype gender '_p' num2str(phase) '.txt'];
    files{i}    = filename;
end

% load simulations: load all files
sims = cell(N,1);
parfor i = 1:N
    sims{i} = load([simfilesfolder '\' files{i}]);
end

% get simulation legnths

APD90s  = zeros(N,1);
APD50s  = APD90s;
APD40s  = APD90s;
RMPs    = APD90s;
maxVs   = APD90s;
maxdVdTs= APD90s;
maxCais = APD90s;
minCais = APD90s;

for i = 1:N
    % get data
    T       = sims{i}(:,1);
    V       = sims{i}(:,2);
    Cai     = sims{i}(:,3);
    % select last beat
    lastB   = T >= (BEATs(i) - 1) * BCLs(i);
    T       = T(lastB);
    V       = V(lastB);
    [APD90s(i), maxdVdTs(i), ~, ~, ~, RMPs(i), maxVs(i)] = getAPD(T,V,90);
    [APD50s(i)] = getAPD(T,V,50);
    [APD40s(i)] = getAPD(T,V,40);
    % save calcium max
    maxCais(i) = max(Cai(lastB));
    minCais(i) = min(Cai(lastB));
end

Tri90_40s = APD90s-APD40s;

% save loaded simulations to a better format
if analize_ToSim_pop == 0
    savename = [population_name(1:end-4) '_sims.mat'];
else
    savename = [population_name(1:end-4) '_toSim_sims.mat'];
end
save(savename,'sims','APD90s','APD50s','APD40s','Tri90_40s','RMPs','maxVs','maxdVdTs','maxCais','minCais','IDs','cells','genders','phases','BCLs','BEATs','SVSTs','SVPs','scalars','channels','-v7.3');

end