%     Copyright (C) 2023 Jordi Cano. Contact: jordi.cg89@gmail.com
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <https://www.gnu.org/licenses/>

function Select_good_simulations_and_plot()
%Select_good_simulations Looks for simulations in the population within the
%range of experimental data.
close all
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
population_name     = 'Population_50000_each.mat';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 0 for endo simulations
% 2 for epi and mid simulations
analize_toSim_pop   = 0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Calibration
APD40_range         = [85           320];      % ms
APD50_range         = [110          350];      % ms
APD90_range         = [180          440];      % ms
Tri90_40_range      = [50           150];      % ms
maxdVdt_range       = [150          1000];     % ms
maxV_range          = [10           55];       % mV
RMP_range           = [-95          -80];      % mV
maxCai_range        = [0.262e-3     2.23e-3];  % mM
minCai_range        = [0            0.40e-3];  % mM

APD90sEPI_range     = [240          340];      % ms 
APD90sMID_range     = [350          450];      % ms 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% prepare things, load sims
pop_sims_name           = [population_name(1:end-4) '_sims.mat'];
right_sim_folder_name   = [population_name(1:end-4) '_plots'];
if analize_toSim_pop == 2
    pop_sims_name       = [population_name(1:end-4) '_toSim_sims.mat'];
    right_sim_folder_name   = [population_name(1:end-4) '_toSim_plots'];
end

disp('- loading matrix')
S                       = load(pop_sims_name);
disp('- loaded')
nosave                  = analize_toSim_pop;


% create important folders
mkdir(right_sim_folder_name);
save_sims_name          = [right_sim_folder_name '\' population_name(1:end-4) '_good.mat'];

if analize_toSim_pop == 0
    inc         = in_range(S.APD90s,APD90_range) & in_range(S.APD50s,APD50_range)...
        & in_range(S.maxdVdTs,maxdVdt_range) & in_range(S.maxVs,maxV_range)...
        & in_range(S.maxCais,maxCai_range) & in_range(S.RMPs,RMP_range)...
        & in_range(S.minCais,minCai_range) & in_range(S.APD40s,APD40_range)...
        & in_range(S.Tri90_40s,Tri90_40_range);
elseif analize_toSim_pop == 1
    inc = ones(size(S.APD90s));
elseif analize_toSim_pop == 2
    inc = S.cells == 2 & in_range(S.APD90s,APD90sMID_range);
end

OU          = ~inc;
NOSEL       = [];

if analize_toSim_pop == 2
    OU          = S.cells == 1 & in_range(S.APD90s,APD90sEPI_range);
    NOSEL       = ~(in_range(S.APD90s,APD90sMID_range) & in_range(S.APD90s,APD90sEPI_range));
end

IN          = inc;
IN_M        = S.genders==1 & IN;
IN_F        = S.genders==2 & IN;
OU_M        = S.genders==1 & OU;
OU_F        = S.genders==2 & OU;

disp('- making figures')
represent(S.scalars, S.APD90s,      IN_M, IN_F, OU_M, OU_F, S.channels, 'APD90',    right_sim_folder_name, nosave, NOSEL);
represent(S.scalars, S.APD50s,      IN_M, IN_F, OU_M, OU_F, S.channels, 'APD50',    right_sim_folder_name, nosave, NOSEL);
represent(S.scalars, S.APD40s,      IN_M, IN_F, OU_M, OU_F, S.channels, 'APD40',    right_sim_folder_name, nosave, NOSEL);
represent(S.scalars, S.Tri90_40s,   IN_M, IN_F, OU_M, OU_F, S.channels, 'Tri90-40', right_sim_folder_name, nosave, NOSEL);
represent(S.scalars, S.maxdVdTs,    IN_M, IN_F, OU_M, OU_F, S.channels, 'maxdVdT',  right_sim_folder_name, nosave, NOSEL);
represent(S.scalars, S.maxVs,       IN_M, IN_F, OU_M, OU_F, S.channels, 'maxV',     right_sim_folder_name, nosave, NOSEL);
represent(S.scalars, S.maxCais,     IN_M, IN_F, OU_M, OU_F, S.channels, 'maxCai',   right_sim_folder_name, nosave, NOSEL);
represent(S.scalars, S.minCais,     IN_M, IN_F, OU_M, OU_F, S.channels, 'minCai',   right_sim_folder_name, nosave, NOSEL);
represent(S.scalars, S.RMPs,        IN_M, IN_F, OU_M, OU_F, S.channels, 'RMP',      right_sim_folder_name, nosave, NOSEL);
disp('- figures done')

disp('- plotting APs')
if analize_toSim_pop ~= 1
    plotAllAPs(S.sims,IN_M,IN_F,OU_M,OU_F,right_sim_folder_name,nosave,NOSEL);
end
disp('- APs plotted and saved (according to pop)')

% supress bad simulations in all parameters
disp('- suppressing excluded simulations')
fields = fieldnames(S);
for ff = 1:length(fields)
    
    fi = fields{ff};
    if ~strcmp(fi,'channels')
        if analize_toSim_pop == 2
            S.(fi) = S.(fi)(IN | OU,:);
        else
            S.(fi) = S.(fi)(IN,:);
        end
    end
    
end

if analize_toSim_pop ~= 1
    disp('- saving matrix')
    save(save_sims_name,'S','-v7.3');
end

disp(' -> All done!')

    function [in_mat] = in_range(matrix,range)
        in_mat = matrix>=range(1) & matrix<=range(2);
    end

    function represent(scalars,y,male,female,exclud_m,exclud_f,channels,lbl,folder,nosave,not_selected)
        % exclud = exclud_m & exclud_f;
        figure('units','normalized','outerposition',[0 0 1 1])
        for i = 1:size(scalars,2)
            subplot(2,6,i); hold on;
            if nosave == 2 && ~isempty(not_selected)
                plot(scalars(not_selected,i),     y(not_selected),      'o',    'color',    [200 200 200]./255,   'markerfacecolor',  [200 200 200]./255,   'markersize',   1.5);
            end
            plot(scalars(exclud_m,i),   y(exclud_m),    'o',    'color',    [153 255 255]./255, 'markerfacecolor',  [153 255 255]./255, 'markersize',   3);
            plot(scalars(exclud_f,i),   y(exclud_f),    'o',    'color',    [255 153 255]./255, 'markerfacecolor',  [255 153 255]./255, 'markersize',   3);
            plot(scalars(male,i),       y(male),        'o',    'color',    [0 102 204]./255,   'markerfacecolor',  [0 102 204]./255,   'markersize',   1.8);
            plot(scalars(female,i),     y(female),      'o',    'color',    [255 0 127]./255,   'markerfacecolor',  [255 0 127]./255,   'markersize',   1.8);
            title(channels{i});
            xlabel('value');
            ylabel(lbl);
            set(gca,'FontName','Times','Fontsize',12);
            if nosave == 0
                if i==1; legend('Excluded (male)','Excluded (female)','Male','Female'); end
            elseif nosave == 1
                if i==1; legend('Male','Female'); end
            elseif nosave == 2
                if ~isempty(not_selected)
                    if i==1; legend('Bad sims (EPI & MID)','EPI Male','EPI Female','ENDO Male','ENDO Female'); end
                else
                    if i==1; legend('EPI Male','EPI Female','ENDO Male','ENDO Female'); end
                end
            end
        end
        if nosave~=1
            saveas(gcf,[folder '\' lbl '.fig']);
        end
    end

    function plotAllAPs(simcell,in_m,in_f,out_m,out_f,folder,nosave,not_selected)
        ds      = 10;
        figure('units','normalized','outerposition',[0 0 1 1]); hold on;
        YES_m   = simcell(in_m);
        YES_f   = simcell(in_f);
        NO_M    = simcell(out_m);
        NO_F    = simcell(out_f);
        NONS    = simcell(not_selected);
        if ~isempty(not_selected)
            for i = 1:length(NONS)
                sim = NONS{i};
                t = downsample(sim(:,1),ds);
                v = downsample(sim(:,2),ds);
                plot(t,v,'color',[200 200 200]./255,'linewidth',1);
            end
        end
        for i = 1:length(NO_M)
            sim = NO_M{i};
            t = downsample(sim(:,1),ds);
            v = downsample(sim(:,2),ds);
            plot(t,v,'color',[153 255 255]./255,'linewidth',1);
        end
        for i = 1:length(NO_F)
            sim = NO_F{i};
            t = downsample(sim(:,1),ds);
            v = downsample(sim(:,2),ds);
            plot(t,v,'color',[255 153 255]./255,'linewidth',1);
        end
        for i = 1:length(YES_m)
            sim = YES_m{i};
            t = downsample(sim(:,1),ds);
            v = downsample(sim(:,2),ds);
            plot(t,v,'color',[0 102 204]./255,'linewidth',1);
        end
        for i = 1:length(YES_f)
            sim = YES_f{i};
            t = downsample(sim(:,1),ds);
            v = downsample(sim(:,2),ds);
            plot(t,v,'color',[255 0 127]./255,'linewidth',1);
        end
        
        if nosave ~= 1
            saveas(gcf,[folder '\' 'APs.fig'],'-v7.3');
        end
    end

end
